@@ -65,23 +65,22 @@ MVMObject * MVM_sc_create(MVMThreadContext *tc, MVMString *handle) {
 /* Makes an entry in all SCs list, the index of which is used to refer to
  * SCs in object headers. This must only be called while holding the SC
  * registry mutex. However, the all SCs list is read without the lock.
- * Thus we allocate memory using the FSA and free it at a safepoint. */
+ * Thus free allocated memory at a safepoint. */
 void MVM_sc_add_all_scs_entry(MVMThreadContext *tc, MVMSerializationContextBody *scb) {
     if (tc->instance->all_scs_next_idx == tc->instance->all_scs_alloc) {
         if (tc->instance->all_scs_next_idx == 0) {
             /* First time; allocate, and NULL first slot as it is
              * the "no SC" sentinel value. */
             tc->instance->all_scs_alloc = 32;
-            tc->instance->all_scs    = MVM_fixed_size_alloc(tc, tc->instance->fsa,
-                tc->instance->all_scs_alloc * sizeof(MVMSerializationContextBody *));
+            tc->instance->all_scs    = MVM_malloc(tc->instance->all_scs_alloc * sizeof(MVMSerializationContextBody *));
             tc->instance->all_scs[0] = NULL;
             tc->instance->all_scs_next_idx++;
         }
         else {
             MVMuint32 orig_alloc = tc->instance->all_scs_alloc;
             tc->instance->all_scs_alloc += 32;
-            tc->instance->all_scs = MVM_fixed_size_realloc_at_safepoint(tc,
-                tc->instance->fsa, tc->instance->all_scs,
+            tc->instance->all_scs = MVM_realloc_at_safepoint(tc,
+                tc->instance->all_scs,
                 orig_alloc * sizeof(MVMSerializationContextBody *),
                 tc->instance->all_scs_alloc * sizeof(MVMSerializationContextBody *));
         }
@@ -92,12 +91,7 @@ void MVM_sc_add_all_scs_entry(MVMThreadContext *tc, MVMSerializationContextBody
 }
 
 void MVM_sc_all_scs_destroy(MVMThreadContext *tc) {
-    MVM_fixed_size_free(
-        tc,
-        tc->instance->fsa,
-        tc->instance->all_scs_alloc * sizeof(MVMSerializationContextBody *),
-        tc->instance->all_scs
-    );
+    MVM_free(tc->instance->all_scs);
 }
 
 /* Given an SC, returns its unique handle. */
@@ -162,8 +156,9 @@ MVMint64 MVM_sc_find_code_idx(MVMThreadContext *tc, MVMSerializationContext *sc,
     roots = sc->body->root_codes;
     count = MVM_repr_elems(tc, roots);
     for (i = 0; i < count; i++) {
-        MVMObject *test = MVM_repr_at_pos_o(tc, roots, i);
-        if (test == obj)
+        MVMRegister test;
+        MVM_VMArray_at_pos(tc, STABLE(roots), roots, OBJECT_BODY(roots), i, &test, MVM_reg_obj);
+        if (test.o == obj)
             return i;
     }
 
@@ -203,7 +198,7 @@ MVMSerializationContext * MVM_sc_get_sc_slow(MVMThreadContext *tc, MVMCompUnit *
 /* Checks if an SC is currently in the process of doing deserialization work. */
 MVM_STATIC_INLINE MVMint64 sc_working(MVMSerializationContext *sc) {
     MVMSerializationReader *sr = sc->body->sr;
-    return sr && sr->working;
+    return sr && MVM_load(&sr->working);
 }
 
 MVMuint8 MVM_sc_is_object_immediately_available(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx) {
@@ -245,6 +240,21 @@ MVMObject * MVM_sc_try_get_object(MVMThreadContext *tc, MVMSerializationContext
         return NULL;
 }
 
+/* Given an SC, an index, and an object, store the object at that index and update
+ * its SC index to match. */
+void MVM_sc_set_object_op(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMObject *obj) {
+    if (REPR(sc)->ID != MVM_REPR_ID_SCRef)
+        MVM_exception_throw_adhoc(tc,
+            "Must provide an SCRef operand to scsetobj");
+    MVM_sc_set_object(tc, sc, idx, obj);
+    if (MVM_sc_get_stable_sc(tc, STABLE(obj)) == NULL) {
+        /* Need to claim the SC also; typical case for new type objects. */
+        MVMSTable *st = STABLE(obj);
+        MVM_sc_set_stable_sc(tc, st, (MVMSerializationContext *)sc);
+        MVM_sc_push_stable(tc, (MVMSerializationContext *)sc, st);
+    }
+}
+
 /* Given an SC, an index, and an object, store the object at that index and update
  * its SC index to match. */
 void MVM_sc_set_object(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMObject *obj) {
@@ -315,10 +325,8 @@ void MVM_sc_set_stable(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint
             sc->body->alloc_stables += 32;
             if (sc->body->alloc_stables < (MVMuint64)idx + 1)
                 sc->body->alloc_stables = idx + 1;
-            sc->body->root_stables = MVM_realloc(sc->body->root_stables,
+            sc->body->root_stables = MVM_recalloc(sc->body->root_stables, orig_size,
                 sc->body->alloc_stables * sizeof(MVMSTable *));
-            memset(sc->body->root_stables + orig_size, 0,
-                (sc->body->alloc_stables - orig_size) * sizeof(MVMSTable *));
         }
         MVM_ASSIGN_REF(tc, &(sc->common.header), sc->body->root_stables[idx], st);
         sc->body->num_stables = idx + 1;
@@ -357,6 +365,14 @@ MVMObject * MVM_sc_get_code(MVMThreadContext *tc, MVMSerializationContext *sc, M
     }
 }
 
+/* Given an SC, an index and a code ref, store it and the index. */
+void MVM_sc_set_code_op(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMObject *code) {
+    if (REPR(sc)->ID != MVM_REPR_ID_SCRef)
+        MVM_exception_throw_adhoc(tc,
+            "Must provide an SCRef operand to scsetcode");
+    MVM_sc_set_obj_sc(tc, code, sc);
+    MVM_sc_set_code(tc, sc, idx, code);
+}
 /* Resolves an SC handle using the SC weakhash. */
 MVMSerializationContext * MVM_sc_find_by_handle(MVMThreadContext *tc, MVMString *handle) {
     uv_mutex_lock(&tc->instance->mutex_sc_registry);
